/*
 * Copyright (c) 2009 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property
 * and proprietary rights in and to this software, related documentation
 * and any modifications thereto.  Any use, reproduction, disclosure or
 * distribution of this software and related documentation without an express
 * license agreement from NVIDIA Corporation is strictly prohibited.
 */

#ifndef __kd_NV_gui_manager_h_
#define __kd_NV_gui_manager_h_

#include <KD/kd.h>
#if !defined(KD_NV_applications) || !KD_NV_applications
#error "NV_gui_manager requires NV_applications"
#endif
#include <KD/NV_applications.h>
#include <EGL/egl.h>
#include <EGL/eglext.h>

#ifdef __cplusplus
extern "C" {
#endif

/* KD GUI objects */
typedef struct KDGuiManagerNV KDGuiManagerNV;
typedef struct KDClientWindowNV KDClientWindowNV;
typedef struct KDClientStreamNV KDClientStreamNV;

/* KD GUI manager functions */
KD_API KDGuiManagerNV* KD_APIENTRY kdCreateGuiManagerNV(void *eventuserptr);
KD_API KDint KD_APIENTRY kdDestroyGuiManagerNV(KDGuiManagerNV *manager);

/* KD GUI state and input management */
KD_API KDint KD_APIENTRY kdRegisterStateiNV(KDint startidx, KDuint numidxs);
KD_API KDint KD_APIENTRY kdRegisterStatelNV(KDint startidx, KDuint numidxs);
KD_API KDint KD_APIENTRY kdRegisterStatefNV(KDint startidx, KDuint numidxs);
KD_API KDint KD_APIENTRY kdManageStateNV(KDGuiManagerNV* manager, KDint startidx, KDuint numidxs);
KD_API KDint KD_APIENTRY kdActualStateGetiNV(KDint startidx, KDuint numidxs, KDint32* buffer);
KD_API KDint KD_APIENTRY kdActualStateGetlNV(KDint startidx, KDuint numidxs, KDint64* buffer);
KD_API KDint KD_APIENTRY kdActualStateGetfNV(KDint startidx, KDuint numidxs, KDfloat32* buffer);
KD_API KDint KD_APIENTRY kdSetManagedStateiNV(KDGuiManagerNV* manager, KDApplicationNV app, KDint startidx, KDuint numidxs, KDint32* values);
KD_API KDint KD_APIENTRY kdSetManagedStatelNV(KDGuiManagerNV* manager, KDApplicationNV app, KDint startidx, KDuint numidxs, KDint64* values);
KD_API KDint KD_APIENTRY kdSetManagedStatefNV(KDGuiManagerNV* manager, KDApplicationNV app, KDint startidx, KDuint numidxs, KDfloat32* values);
KD_API KDint KD_APIENTRY kdPostFocusEventNV(KDEvent* event);

/* KD GUI state management */
KD_API KDint KD_APIENTRY kdRegisterOutputiNV(KDint startidx, KDuint numidxs);
KD_API KDint KD_APIENTRY kdRegisterOutputfNV(KDint startidx, KDuint numidxs);
KD_API KDint KD_APIENTRY kdManageOutputNV(KDGuiManagerNV* manager, KDint startidx, KDuint numidxs);
KD_API KDint KD_APIENTRY kdActualOutputSetiNV(KDint startidx, KDuint numidxs, const KDint32* buffer);
KD_API KDint KD_APIENTRY kdActualOutputSetfNV(KDint startidx, KDuint numidxs, const KDfloat32* buffer);

/* KD GUI window management */
KD_API KDint KD_APIENTRY kdRegisterWindowPropertybNV(KDint pname, KDint count);
KD_API KDint KD_APIENTRY kdRegisterWindowPropertyiNV(KDint pname, KDint count);
KD_API KDint KD_APIENTRY kdRegisterWindowPropertycNV(KDint pname);
KD_API KDint KD_APIENTRY kdManageWindowsNV(KDGuiManagerNV* manager, EGLDisplay display, const KDchar* desktop);
KD_API KDClientWindowNV* KD_APIENTRY kdAcceptClientWindowNV(KDGuiManagerNV* manager, void* eventuserptr);
KD_API KDint KD_APIENTRY kdCloseClientWindowNV(KDClientWindowNV* window);
KD_API KDint KD_APIENTRY kdPostClientWindowEventNV(KDEvent* event, KDClientWindowNV* window);
KD_API KDApplicationNV KD_APIENTRY kdGetClientWindowApplicationNV(KDClientWindowNV* window);
KD_API KDint KD_APIENTRY kdGetRequestedClientWindowPropertybvNV(KDClientWindowNV* window, KDint pname, KDboolean *param);
KD_API KDint KD_APIENTRY kdGetRequestedClientWindowPropertyivNV(KDClientWindowNV* window, KDint pname, KDint32 *param);
KD_API KDint KD_APIENTRY kdGetRequestedClientWindowPropertycvNV(KDClientWindowNV* window, KDint pname, KDchar *param, KDsize *size);
KD_API KDint KD_APIENTRY kdGetCurrentClientWindowPropertybvNV(KDClientWindowNV* window, KDint pname, KDboolean *param);
KD_API KDint KD_APIENTRY kdGetCurrentClientWindowPropertyivNV(KDClientWindowNV* window, KDint pname, KDint32 *param);
KD_API KDint KD_APIENTRY kdGetCurrentClientWindowPropertycvNV(KDClientWindowNV* window, KDint pname, KDchar *param, KDsize *size);
KD_API KDint KD_APIENTRY kdSetClientWindowPropertybvNV(KDClientWindowNV* window, KDint pname, const KDboolean *param);
KD_API KDint KD_APIENTRY kdSetClientWindowPropertyivNV(KDClientWindowNV* window, KDint pname, const KDint32 *param);
KD_API KDint KD_APIENTRY kdSetClientWindowPropertycvNV(KDClientWindowNV* window, KDint pname, const KDchar *param);
KD_API KDint KD_APIENTRY kdUnlockClientWindowBufferNV(KDClientWindowNV* window, KDint index, EGLenum api);
KD_API KDint KD_APIENTRY kdLockClientWindowBufferNV(KDClientWindowNV* window);

/* KD GUI stream management */
KD_API KDint KD_APIENTRY kdManageStreamsNV(KDGuiManagerNV* manager, EGLDisplay display, const KDchar* desktop);
KD_API KDClientStreamNV* KD_APIENTRY kdAcceptClientStreamNV(KDGuiManagerNV* manager, void* eventuserptr);
KD_API KDint KD_APIENTRY kdCloseClientStreamNV(KDClientStreamNV* stream);
KD_API KDApplicationNV KD_APIENTRY kdGetClientStreamApplicationNV(KDClientStreamNV* stream);
KD_API KDint KD_APIENTRY kdGetClientStreamPropertybvNV(KDClientStreamNV* stream, KDint pname, KDboolean* param);
KD_API KDint KD_APIENTRY kdGetClientStreamPropertyivNV(KDClientStreamNV* stream, KDint pname, KDint32* param);
KD_API KDint KD_APIENTRY kdGetClientStreamPropertyfvNV(KDClientStreamNV* stream, KDint pname, KDfloat32* param);
KD_API KDint KD_APIENTRY kdGetClientStreamPropertycvNV(KDClientStreamNV* stream, KDint pname, KDchar* param, KDsize* size);
KD_API KDint KD_APIENTRY kdUnlockClientStreamBufferNV(KDClientStreamNV* stream, KDint index, EGLenum api);
KD_API KDint KD_APIENTRY kdLockClientStreamBufferNV(KDClientStreamNV* stream);



/* KD GUI manager event codes (TODO: Need an official set of values) */

#define KD_EVENT_GUI_MANAGEDSTATE_NV        6003

#define KD_EVENT_GUI_MANAGEDOUTPUT_NV       6004

#define KD_EVENT_GUI_WINDOWAVAILABLE_NV     6005
#define KD_EVENT_GUI_WINDOWCLOSED_NV        6006
#define KD_EVENT_GUI_WINDOWPROPERTY_NV      6007
#define KD_EVENT_GUI_WINDOWBUFFERS_NV       6008
#define KD_EVENT_GUI_WINDOWSWAP_NV          6009
#define KD_EVENT_GUI_WINDOWLOCK_NV          6010

#define KD_EVENT_GUI_STREAMAVAILABLE_NV     6011
#define KD_EVENT_GUI_STREAMCLOSED_NV        6012
#define KD_EVENT_GUI_STREAMPROPERTY_NV      6013
#define KD_EVENT_GUI_STREAMBUFFERS_NV       6014
#define KD_EVENT_GUI_STREAMSWAP_NV          6015
#define KD_EVENT_GUI_STREAMLOCK_NV          6016

/* KD GUI window properties */
#define KD_WINDOWPROPERTY_MANAGEABLE_NV     6017

/* KD GUI stream properties */
#define KD_STREAMPROPERTY_SIZE_NV           6020

#ifdef __cplusplus
}
#endif

#endif /* __kd_NV_gui_manager_h_ */
