/*
 * Copyright (c) 2011 NVIDIA Corporation.  All rights reserved.
 *
 * NVIDIA Corporation and its licensors retain all intellectual property
 * and proprietary rights in and to this software, related documentation
 * and any modifications thereto.  Any use, reproduction, disclosure or
 * distribution of this software and related documentation without an express
 * license agreement from NVIDIA Corporation is strictly prohibited.
 */

#ifndef __KD_NV_native_pixmap_h
#define __KD_NV_native_pixmap_h

#include <KD/kd.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Pixmap type */
typedef struct KDPixmapNV KDPixmapNV;

/* Map attributes */
#define KD_PIXMAP_MAP_PRESERVE_PIXELS_NV           366
#define KD_PIXMAP_MAP_USAGE_HINT_NV                367
#define KD_READ_PIXMAP_BIT_NV               0x00000001
#define KD_WRITE_PIXMAP_BIT_NV              0x00000002

/* Pixmap properties */
#define KD_PIXMAPPROPERTY_SIZE_NV                  372
#define KD_PIXMAPPROPERTY_POINTER_NV               373
#define KD_PIXMAPPROPERTY_PHYSICAL_POINTER_NV      374
#define KD_PIXMAPPROPERTY_PITCH_NV                 375
#define KD_PIXMAPPROPERTY_ORIGIN_NV                376
#define KD_PIXMAPPROPERTY_PIXEL_SIZE_NV            377
#define KD_PIXMAPPROPERTY_RED_OFFSET_NV            378
#define KD_PIXMAPPROPERTY_GREEN_OFFSET_NV          379
#define KD_PIXMAPPROPERTY_BLUE_OFFSET_NV           380
#define KD_PIXMAPPROPERTY_ALPHA_OFFSET_NV          381
#define KD_PIXMAPPROPERTY_LUMINANCE_OFFSET_NV      382

#define KD_PIXMAP_POINTER_SIZE_NV                    1
#define KD_PIXMAP_LOWER_LEFT_NV                      1
#define KD_PIXMAP_UPPER_LEFT_NV                      2

/* Pixmap functions */
KD_API KDPixmapNV* KD_APIENTRY kdCreatePixmapNV(EGLDisplay display, EGLConfig config);
KD_API KDint KD_APIENTRY kdDestroyPixmapNV(KDPixmapNV* pixmap);

KD_API KDint KD_APIENTRY kdSetPixmapPropertybvNV(KDPixmapNV* pixmap, KDint pname, const KDboolean* param);
KD_API KDint KD_APIENTRY kdSetPixmapPropertyivNV(KDPixmapNV* pixmap, KDint pname, const KDint32* param);
KD_API KDint KD_APIENTRY kdSetPixmapPropertycvNV(KDPixmapNV* pixmap, KDint pname, const KDchar* param);

KD_API KDint KD_APIENTRY kdGetPixmapPropertybvNV(KDPixmapNV* pixmap, KDint pname, KDboolean* param);
KD_API KDint KD_APIENTRY kdGetPixmapPropertyivNV(KDPixmapNV* pixmap, KDint pname, KDint32* param);
KD_API KDint KD_APIENTRY kdGetPixmapPropertycvNV(KDPixmapNV* pixmap, KDint pname, KDchar* param, KDsize* size);

KD_API KDint KD_APIENTRY kdRealizePixmapNV(KDPixmapNV* pixmap, EGLNativePixmapType* nativepixmap);

KD_API KDint KD_APIENTRY kdMapPixmapNV(KDPixmapNV* pixmap, const KDint* attrib_list);
KD_API KDint KD_APIENTRY kdUnmapPixmapNV(KDPixmapNV* pixmap);


#ifdef __cplusplus
}
#endif

#endif /* __KD_NV_native_pixmap_h_ */
