#include <assert.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include "GLES2/gl2.h"

#include "priv.h"
#include "nvtexfont2.h"

#include "inconsolata.h"

int nvtexfont2InVectorFont(NVTexfont2VectorFont * vtf, int c)
{
    return (vtf->offsets[c] > -1);
}

NVTexfont2VectorFont *nvtexfont2InitVectorFont(NVTexfont2VectorFontName font,
                                         GLboolean     antialias,
                                         GLboolean     use_vbo)
{
    GLint savedArrayBufferBinding;
    NVTexfont2VectorFont *vtf = malloc(sizeof(NVTexfont2VectorFont));

    if (vectorProg == 0)
        initShaders();

    switch (font)
    {
    case NV_VECTOR_TEXFONT_INCONSOLATA:
        vtf->vertices = NVTEXFONT_inconsolata_VERTS;
        vtf->offsets = NVTEXFONT_inconsolata_OFFSETS;
        vtf->counts = NVTEXFONT_inconsolata_COUNTS;
        vtf->widths = NVTEXFONT_inconsolata_WIDTHS;
        vtf->vertNo = NVTEXFONT_inconsolata_VERTNO;

        break;
    default:
        goto initVectorError;
    }
    vtf->antialias = antialias;

    if (use_vbo == GL_TRUE)
    {

        glGetIntegerv(GL_ARRAY_BUFFER_BINDING, &savedArrayBufferBinding);

        glGenBuffers(1, &vtf->vbo);
        glBindBuffer(GL_ARRAY_BUFFER, vtf->vbo);
        glBufferData(GL_ARRAY_BUFFER, vtf->vertNo * 2, vtf->vertices, GL_STATIC_DRAW);

        glBindBuffer(GL_ARRAY_BUFFER, savedArrayBufferBinding);
    }
    else
    {
        vtf->vbo = -1;
    }

    return vtf;
initVectorError:
    free(vtf);
    return 0;
}

void nvtexfont2UnloadVectorFont(NVTexfont2VectorFont *vtf)
{
    deinitShaders();

    if (vtf->vbo != -1)
    {
        glDeleteBuffers(1, &vtf->vbo);
    }

    free(vtf);
}

static int renderVecGlyph(NVTexfont2VectorFont *vtf, int c)
{
    if (vtf->offsets[c] == -1)
        return vtf->widths[c];

    if (vtf->antialias)
    {
        /* CURRENTLY UNIMPLEMENTED */
    }

    glDrawArrays(GL_TRIANGLES, vtf->offsets[c], vtf->counts[c]);

    return vtf->widths[c];
}

void nvtexfont2RenderVecString_All(NVTexfont2VectorFont * vtf, char *string,
                                              float x, float y,
                                              float scale,
                                              float r, float g, float b)
{
    GLboolean savedBlendEnabled, savedDepthTestEnabled, savedCullFaceEnabled;
    GLint savedBlendSrcRGB, savedBlendSrcAlpha, savedBlendDstRGB, savedBlendDstAlpha;
    GLint savedVertArrayBinding, savedVertArrayEnabled, savedVertArraySize;
    GLint savedVertArrayStride, savedVertArrayType, savedVertArrayNormalized;
    void *savedVertArrayPointer;
    GLint savedCurrentProgram;
    GLint savedArrayBufferBinding;

    GLuint uniScale, uniColor, uniScreenPos, uniOffset;

    int offset = 0, i = 0;

    /*****************
     * SAVE GL STATE *
     *****************/

    savedBlendEnabled = glIsEnabled(GL_BLEND);
    savedDepthTestEnabled = glIsEnabled(GL_DEPTH_TEST);
    savedCullFaceEnabled = glIsEnabled(GL_CULL_FACE);

    glGetIntegerv(GL_BLEND_SRC_RGB, &savedBlendSrcRGB);
    glGetIntegerv(GL_BLEND_SRC_ALPHA, &savedBlendSrcAlpha);
    glGetIntegerv(GL_BLEND_DST_RGB, &savedBlendDstRGB);
    glGetIntegerv(GL_BLEND_DST_ALPHA, &savedBlendDstAlpha);

    glGetVertexAttribiv(VERT_ARRAY, GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING, &savedVertArrayBinding);
    glGetVertexAttribiv(VERT_ARRAY, GL_VERTEX_ATTRIB_ARRAY_ENABLED, &savedVertArrayEnabled);
    glGetVertexAttribiv(VERT_ARRAY, GL_VERTEX_ATTRIB_ARRAY_SIZE, &savedVertArraySize);
    glGetVertexAttribiv(VERT_ARRAY, GL_VERTEX_ATTRIB_ARRAY_STRIDE, &savedVertArrayStride);
    glGetVertexAttribiv(VERT_ARRAY, GL_VERTEX_ATTRIB_ARRAY_TYPE, &savedVertArrayType);
    glGetVertexAttribiv(VERT_ARRAY, GL_VERTEX_ATTRIB_ARRAY_NORMALIZED, &savedVertArrayNormalized);
    glGetVertexAttribPointerv(VERT_ARRAY, GL_VERTEX_ATTRIB_ARRAY_POINTER, &savedVertArrayPointer);

    glGetIntegerv(GL_CURRENT_PROGRAM, &savedCurrentProgram);

    /************
     * SETUP GL *
     ************/

    glEnable(GL_BLEND);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_CULL_FACE);
    glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glUseProgram(vectorProg);

    glGetIntegerv(GL_ARRAY_BUFFER_BINDING, &savedArrayBufferBinding);

    glEnableVertexAttribArray(VERT_ARRAY);

    if (vtf->vbo == -1)
    {
        glVertexAttribPointer(VERT_ARRAY, 2, GL_UNSIGNED_BYTE, GL_FALSE, 0, vtf->vertices);
    }
    else
    {
        glBindBuffer(GL_ARRAY_BUFFER, vtf->vbo);
        glVertexAttribPointer(VERT_ARRAY, 2, GL_UNSIGNED_BYTE, GL_FALSE, 0, 0);
    }

    /*****************
     * SETUP SHADERS *
     *****************/

    uniScale = glGetUniformLocation(vectorProg, "scale");
    uniColor = glGetUniformLocation(vectorProg, "color");
    uniScreenPos = glGetUniformLocation(vectorProg, "screenPos");

    // Adjust scale so that character size is approximately 1/10th screen height
    glUniform1f(uniScale, 0.002f * scale);
    glUniform3f(uniColor, r, g, b);
    glUniform2f(uniScreenPos, x, y);

    uniOffset = glGetUniformLocation(vectorProg, "offset");

    /*************
     * DRAW TEXT *
     *************/

    while (string[i]) {
        glUniform1i(uniOffset, offset);
        offset += renderVecGlyph(vtf, string[i]);
        i++;
    }

    /********************
     * RESTORE GL STATE *
     ********************/

    glBindBuffer(GL_ARRAY_BUFFER, savedArrayBufferBinding);
    glUseProgram(savedCurrentProgram);


    if (!savedVertArrayEnabled)
        glDisableVertexAttribArray(VERT_ARRAY);

    glVertexAttribPointer(VERT_ARRAY, savedVertArraySize, savedVertArrayType, savedVertArrayNormalized,
                          savedVertArrayStride, savedVertArrayPointer);

    if (!savedBlendEnabled)
        glDisable(GL_BLEND);
    glBlendFuncSeparate(savedBlendSrcRGB, savedBlendDstRGB, savedBlendSrcAlpha, savedBlendDstAlpha);

    if (savedDepthTestEnabled)
        glEnable(GL_DEPTH_TEST);

    if (savedCullFaceEnabled)
        glEnable(GL_CULL_FACE);
}

void nvtexfont2RenderVecString_Pos(NVTexfont2VectorFont * vtf, NVTexfont2Context tfc,
                                   char *string,
                                   float x, float y)
{
    nvtexfont2RenderVecString_All(vtf, string, x, y, ((nvtexfont2State *)tfc)->scale, ((nvtexfont2State *)tfc)->r, ((nvtexfont2State *)tfc)->g, ((nvtexfont2State *)tfc)->b);
}

void nvtexfont2RenderVecString(NVTexfont2VectorFont * vtf, NVTexfont2Context tfc,
                               char *string)
{
    nvtexfont2RenderVecString_All(vtf, string, ((nvtexfont2State *)tfc)->x, ((nvtexfont2State *)tfc)->y, ((nvtexfont2State *)tfc)->scale, ((nvtexfont2State *)tfc)->r, ((nvtexfont2State *)tfc)->g, ((nvtexfont2State *)tfc)->b);
}


